# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

import datetime
import time

import requests

from meican.commands import get_dishes, get_restaurants, get_tabs
from meican.exceptions import MeiCanError, NoOrderAvailable
from meican.models import TabStatus
from meican.utils import join_dict


class RestUrl(object):
    """ 用来存储 MeiCan Rest 接口的类 """

    @classmethod
    def get_base_url(cls, path, params=None, wrap=True):
        """
        :type path: str | unicode
        :type params: dict
        :type wrap: bool
        :rtype: str | unicode
        """
        if params:
            if wrap:
                params['noHttpGetCache'] = int(time.time() * 1000)
            path = '{}?{}'.format(path, join_dict(params))
        return 'https://meican.com/{}'.format(path)

    @classmethod
    def login(cls):
        return cls.get_base_url('account/directlogin')

    @classmethod
    def calender_items(cls, detail=False):
        today = datetime.date.today()
        one_week = datetime.timedelta(weeks=1)
        data = {
            'beginDate': today.strftime('%Y-%m-%d'),
            'endDate': (today + one_week).strftime('%Y-%m-%d'),
            'withOrderDetail': detail,
        }
        return cls.get_base_url('preorder/api/v2.1/calendarItems/list', data)

    @classmethod
    def restaurants(cls, tab):
        """
        :type tab: meican.models.Tab
        """
        data = {
            'tabUniqueId': tab.uid,
            'targetTime': tab.target_time,
        }
        return cls.get_base_url('preorder/api/v2.1/restaurants/list', data)

    @classmethod
    def dishes(cls, restaurant):
        """
        :type restaurant: meican.models.Restaurant
        """
        tab = restaurant.tab
        data = {
            'restaurantUniqueId': restaurant.uid,
            'tabUniqueId': tab.uid,
            'targetTime': tab.target_time,
        }
        return cls.get_base_url('preorder/api/v2.1/restaurants/show', data)

    @classmethod
    def order(cls, dish):
        """
        :type dish: meican.models.Dish
        """
        tab = dish.restaurant.tab
        address = tab.addresses[0]  # todo select address or error
        data = {
            'order': json.dumps([{
                'count': '1',
                'dishId': '{}'.format(dish.id),
            }]),
            'tabUniqueId': tab.uid,
            'targetTime': tab.target_time,
            'corpAddressUniqueId': address.uid,
            'userAddressUniqueId': address.uid,
        }
        return cls.get_base_url('preorder/api/v2.1/orders/add', data, wrap=False)


class MeiCan(object):
    def __init__(self, username, password):
        """
        :type username: str | unicode
        :type password: str | unicode
        """
        self.responses = []
        self._session = requests.Session()
        self._calendar_items = None
        self._tabs = None

        form_data = {'username': username, 'password': password, 'loginType': 'username', 'remember': True}
        response = self._request('post', RestUrl.login(), form_data)
        if 200 != response.status_code:  # or '用户名或密码错误' in response.content:
            raise MeiCanError('login fail because username or password incorrect')

    @property
    def tabs(self):
        """
        :rtype: list[meican.models.Tab]
        """
        if not self._tabs:
            self.load_tabs()
        return self._tabs

    @property
    def next_available_tab(self):
        """
        :rtype: meican.models.Tab
        """
        available_tabs = [_ for _ in self.tabs if _.status == TabStatus.AVAIL]
        return available_tabs[0] if available_tabs else None

    def load_tabs(self, refresh=False):
        if not self._calendar_items or refresh:
            self._calendar_items = self.http_get(RestUrl.calender_items())
            self._tabs = get_tabs(self._calendar_items)

    def get_restaurants(self, tab):
        """
        :type tab: meican.models.Tab
        :rtype: list[meican.models.Restaurant]
        """
        data = self.http_get(RestUrl.restaurants(tab))
        return get_restaurants(tab, data)

    def get_dishes(self, restaurant):
        """
        :type restaurant: meican.models.Restaurant
        """
        data = self.http_get(RestUrl.dishes(restaurant))
        return get_dishes(restaurant, data)

    def list_dishes(self, tab=None):
        """
        :type tab: meican.models.Tab
        :rtype: list[meican.models.Dish]
        """
        tab = tab or self.next_available_tab
        if not tab:
            raise NoOrderAvailable('Currently no available orders')
        restaurants = self.get_restaurants(tab)
        dishes = []
        for restaurant in restaurants:
            dishes.extend(self.get_dishes(restaurant))
        return dishes

    def order(self, dish):
        """
        :type dish: meican.models.Dish
        """
        data = self.http_post(RestUrl.order(dish))
        return data

    def http_get(self, url, **kwargs):
        """
        :type url: str | unicode
        :rtype: dict | str | unicode
        """
        response = self._request('get', url, **kwargs)
        return response.json()

    def http_post(self, url, data=None, **kwargs):
        """
        :type url: str | unicode
        :type data: dict
        :rtype: dict | str | unicode
        """
        response = self._request('post', url, data, **kwargs)
        return response.json()

    def _request(self, method, url, data=None, **kwargs):
        """
        :type method: str | unicode
        :type url: str | unicode
        :type data: dict
        :type kwargs: dict
        :rtype: requests.Response
        """
        func = getattr(self._session, method)
        response = func(url, data=data, **kwargs)  # type: requests.Response
        response.encoding = response.encoding or 'utf-8'
        self.responses.append(response)
        if response.status_code != 200:
            error = response.json()
            raise MeiCanError('[{}] {}'.format(error.get('error', ''), error.get('error_description', '')))
        return response



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

from datetime import datetime

from enum import Enum


class TabStatus(Enum):
    AVAIL = 'AVAILABLE'
    CLOSED = 'CLOSED'
    NOT_YET = 'NOT_YET'
    UNKNOWN = 'UNKNOWN'
    ORDERED = 'ORDERED'

    @classmethod
    def parse(cls, string_value):
        """
        :type string_value: str | unicode
        :rtype: TabStatus
        """
        value_enums = {
            'AVAILABLE': cls.AVAIL,
            'CLOSED': cls.CLOSED,
            'NOT_YET': cls.NOT_YET,
            'ORDER': cls.ORDERED,
        }
        return value_enums.get(string_value, cls.UNKNOWN)


class ReadableObject(object):
    def __unicode__(self):
        return '<{}>'.format(self.__class__)

    def __repr__(self):
        output = '{}'.format(self.__unicode__())
        if isinstance(output, unicode):
            output = output.encode('utf8')
        return output


class Address(ReadableObject):
    """ 地址 """

    def __init__(self, data):
        """
        :type data: dict
        """
        self.uid = data['uniqueId']
        self.address = data['address']  # 公司地址
        self.pick_up = data['pickUpLocation']  # 取餐地址

    def __unicode__(self):
        return '{} {}'.format(self.uid, self.address)


class Tab(ReadableObject):
    """
    一个 Tab 代表一个时间窗口
    比如 中饭点餐时间
    或者 晚饭点餐时间
    """

    def __init__(self, data):
        """
        :type data: dict
        """
        self.title = data['title']
        self.target_time = datetime.fromtimestamp(int(data['targetTime']) / 1000)
        self.status = TabStatus.parse(data['status'])
        self.uid = data['userTab']['uniqueId']
        self.addresses = [Address(_) for _ in data['userTab']['corp']['addressList']]

    def __unicode__(self):
        return '{} {} {}'.format(self.status.value, self.target_time.strftime('%Y-%m-%d'), self.title)


class Restaurant(ReadableObject):
    """ 餐厅 """

    def __init__(self, tab, data):
        """
        :type tab: Tab
        :type data: dict
        """
        self.tab = tab
        self.uid = data['uniqueId']
        self.name = data['name']
        self.is_open = data['open']
        self.rating = data['rating']
        self.tel = data['tel']
        self.latitude = data['latitude']
        self.longitude = data['longitude']

    def __unicode__(self):
        return '{}'.format(self.name)


class Dish(ReadableObject):
    """ 菜 """

    def __init__(self, restaurant, data):
        """
        :type data: dict
        :type restaurant: Restaurant
        """
        self.restaurant = restaurant
        self.id = int(data['id'])
        self.name = data['name']
        self.price = data['priceString']

    def __unicode__(self):
        return '{}'.format(self.name)



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals


class MeiCanError(Exception):
    """
    啊，又出错了
    """
    pass


class MeiCanKeyError(MeiCanError):
    """
    MeiCan 接口改了

    要么就是哪里写了屎代码

    反正就是 dict key 不对
    """
    pass


class NoOrderAvailable(MeiCanError):
    """ 目前还点不了餐 """
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

import argparse
import os
import sys

sys.path.append(os.path.join(os.path.dirname(__file__), '..'))

from meican.exceptions import NoOrderAvailable
from meican.settings import MeiCanSetting
from meican.tools import MeiCan


def initialize_meican():
    settings = MeiCanSetting()
    settings.load_credentials()
    return MeiCan(settings.username, settings.password)


def execute(argv=None):
    if argv is None:
        argv = sys.argv[1:] or []
    parser = argparse.ArgumentParser(description='命令行点美餐的工具')
    parser.add_argument('-o', '--order', help='order meal')
    args = parser.parse_args(argv)

    meican = initialize_meican()

    try:
        dishes = meican.list_dishes()
    except NoOrderAvailable:
        print('别急，下一顿还没开放订餐')
        return
    if args.order:
        keyword = args.order.decode('utf-8')
        dishes = [_ for _ in dishes if keyword in _.name]
        if len(dishes) == 1:
            meican.order(dishes[0])
            print('done!')
        elif not dishes:
            print('没有找到 {} 的对应菜品'.format(keyword))
        else:
            print('找到多于一个菜品，请指定更详细的关键词')
            print('\n'.join(['{}'.format(_) for _ in dishes]))


if __name__ == '__main__':
    execute()



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

import datetime
import json


def milli_to_datetime(milliseconds):
    return datetime.date(1970, 1, 1) + datetime.timedelta(milliseconds=int(milliseconds))


def json_dump(data):
    return json.dumps(data, ensure_ascii=False, indent=2)


def join_dict(data, combinator='=', separator='&'):
    """
    :type data: dict
    :type combinator: str | unicode
    :type separator: str | unicode
    :rtype: str | unicode
    """
    return separator.join(['{}{}{}'.format(k, combinator, v) for k, v in sorted(data.items())])


# noinspection PyCompatibility
def prompt(hint):
    try:
        return raw_input(hint)
    except NameError:
        return input(hint)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import json
from os.path import exists, expanduser, join

from meican.utils import json_dump, prompt

setting_file = join(expanduser('~'), '.meicanrc')


class MeiCanSetting(object):
    def __init__(self):
        self._settings = {}
        if not exists(setting_file):
            return
        with open(setting_file) as f:
            self._settings = json.load(f)

    def save(self):
        with open(setting_file, str('w')) as f:
            f.write(json_dump(self._settings))

    def load_credentials(self):
        for key in ['username', 'password']:
            if key not in self._settings:
                self._settings[key] = prompt('please input your meican {}: '.format(key))
        self.save()

    def __getattr__(self, item):
        return self._settings[item]



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

from meican.models import Restaurant, Tab, Dish


def get_tabs(data):
    """
    :type data: dict
    :rtype: list[meican.models.Tab]
    """
    tabs = []
    for day in data['dateList']:
        tabs.extend([Tab(_) for _ in day['calendarItemList']])
    return tabs


def get_restaurants(tab, data):
    """
    :type tab: meican.models.Tab
    :type data: dict
    :rtype: list[meican.models.Restaurant]
    """
    restaurants = []
    for restaurant_data in data['restaurantList']:
        restaurants.append(Restaurant(tab, restaurant_data))
    return restaurants


def get_dishes(restaurant, data):
    """
    :type restaurant: meican.models.Restaurant
    :type data: dict
    :rtype: list[meican.models.Dish]
    """
    dishes = []
    for dish_data in data['dishList']:
        if dish_data.get('isSection', False) or dish_data.get('priceString') is None:
            continue
        dishes.append(Dish(restaurant, dish_data))
    return dishes
